/*
 * adapted from /usr/include/elf.h
 */
const int EI_NIDENT = 16;

alias ushort Elf32_Half;
alias ushort Elf32_Section;
alias uint Elf32_Word;
alias uint Elf32_Addr;
alias uint Elf32_Off;

struct Elf32_Ehdr
{
  ubyte	e_ident[EI_NIDENT];	/* Magic number and other info */
  Elf32_Half	e_type;			/* Object file type */
  Elf32_Half	e_machine;		/* Architecture */
  Elf32_Word	e_version;		/* Object file version */
  Elf32_Addr	e_entry;		/* Entry point virtual address */
  Elf32_Off	e_phoff;		/* Program header table file offset */
  Elf32_Off	e_shoff;		/* Section header table file offset */
  Elf32_Word	e_flags;		/* Processor-specific flags */
  Elf32_Half	e_ehsize;		/* ELF header size in bytes */
  Elf32_Half	e_phentsize;		/* Program header table entry size */
  Elf32_Half	e_phnum;		/* Program header table entry count */
  Elf32_Half	e_shentsize;		/* Section header table entry size */
  Elf32_Half	e_shnum;		/* Section header table entry count */
  Elf32_Half	e_shstrndx;		/* Section header string table index */
}

const ubyte ELFMAG[4] = cast(ubyte[4])"\177ELF";

const int EI_CLASS = 4;		/* File class byte index */
const int ELFCLASS32 = 1;		/* 32-bit objects */

const int EI_DATA = 5; /* Data encoding byte index */
const int ELFDATA2LSB = 1;		/* 2's complement, little endian */
const int ELFDATA2MSB = 2;		/* 2's complement, big endian */

struct Elf32_Shdr
{
  Elf32_Word	sh_name;		/* Section name (string tbl index) */
  Elf32_Word	sh_type;		/* Section type */
  Elf32_Word	sh_flags;		/* Section flags */
  Elf32_Addr	sh_addr;		/* Section virtual addr at execution */
  Elf32_Off	sh_offset;		/* Section file offset */
  Elf32_Word	sh_size;		/* Section size in bytes */
  Elf32_Word	sh_link;		/* Link to another section */
  Elf32_Word	sh_info;		/* Additional section information */
  Elf32_Word	sh_addralign;		/* Section alignment */
  Elf32_Word	sh_entsize;		/* Entry size if section holds table */
}

const SHT_SYMTAB = 2;		/* Symbol table */
const SHT_STRTAB = 3;		/* String table */
const SHN_UNDEF	= 0;		/* Undefined section */
const SHF_ALLOC	= (1 << 1);	/* Occupies memory during execution */

struct Elf32_Sym
{
  Elf32_Word	st_name;		/* Symbol name (string tbl index) */
  Elf32_Addr	st_value;		/* Symbol value */
  Elf32_Word	st_size;		/* Symbol size */
  ubyte	st_info;		/* Symbol type and binding */
  ubyte st_other;		/* Symbol visibility */
  Elf32_Section	st_shndx;		/* Section index */
}

const STT_NOTYPE = 0;		/* Symbol type is unspecified */
const STT_OBJECT = 1;		/* Symbol is a data object */
const STT_FUNC = 2;		/* Symbol is a code object */
const STT_SECTION = 3;		/* Symbol associated with a section */
const STT_FILE = 4;		/* Symbol's name is file name */
const STT_COMMON = 5;		/* Symbol is a common data object */
const STT_TLS = 6;		/* Symbol is thread-local data object*/

int ELF32_ST_TYPE(Elf32_Word st_info) { return st_info & 0xf; }
