import std.stream;
import elf;

class ElfStrTab
{
    private char tab[];
    
    this(char tab[]) { this.tab = tab; }

    static ElfStrTab readFromStream(Stream stream, Elf32_Shdr shdr)
    {
	if(shdr.sh_type != SHT_STRTAB)
	{
	    throw new Error("attempt to create a string table from a non-SHT_STRTAB section");
	}
	stream.position(shdr.sh_offset);
	char[] tab = new char[shdr.sh_size];
	stream.readExact(&tab[0], tab.length);

	return new ElfStrTab(tab);
    }

    string get(int pos)
    {
	if(pos < 0 || pos >= tab.length)
	{
	    throw new Error("strtab index out of range");
	}
	int endpos = pos;
	while(tab[endpos] && endpos < tab.length)
	{
	    ++endpos;
	}
	return cast(string)(tab[pos .. endpos]);
    }
}
